# import the built-in modules
import MDM
import MOD
import SER2
import GPIO

GPS_PORT_SPEED = '4800' # 9600 with new firmware for GPS
GPIO_BOOT_SEL = 5
GPIO_RESET = 6
GPIO_POWER_ON = 7

class Register:
	"Main register used by the application."
	all_values = {'def_recipient_num' : '3348805023', 'recipient_number':'', 'command_to_execute':'' , 'go_to_sleep_flag' : 0, 'shdwn_time' : 0, 'GSM_reg_status' : '', 'SMS_settings' : '', 'wake_up_interval' : 60}
	def clear_command(self):
		"Reset command_to_execute value"
		self.all_values['command_to_execute'] = ''

	def clear_recipient_number(self):
		"Reset recipient number"
		self.all_values['recipient_number'] = ''


class SMS:
	"Incoming and outcoming SMS format"
	pass

class NMEAsentence:
	"General message format coming from GPS module"
	pass

class NMEAparser:
	"Find a specific pattern in a string"
	def getLastRMCsentence(self, raw_data, sentences_buffer):
		printI('Parser.getLastRMCsentence')
		sentences_buffer['lastRMC'] = ''
		ret_value = -1
		head_index = raw_data.rfind('$GPRMC,')
		if head_index != -1:
			# Valid header has been found!
			trailer_index = raw_data.find('\015',head_index)
			if trailer_index != -1:
				# Valid trailer has been found!
				sentences_buffer['lastRMC'] = raw_data[head_index:trailer_index]
				ret_value = 0
				printI('Valid sentence has been found!')
			else:
				printW(raw_data)
				printW('Valid trailer not found!')
		else:
			printW(raw_data)
			printW('Valid header not found!')
		return ret_value

class GPSManager:
	"Initialize and configure GPS module"
	def GPSpowerOn(self):
		"Power on the GPS"
		ret_value = -1
		res = -1
		# At first set the boot sel pin
		res = GPIO.setIOdir(GPIO_BOOT_SEL, 0, 1)
		if res == 1:
			# Set the reset pin
			res = GPIO.setIOdir(GPIO_RESET, 0, 1)
			if res == 1:
				# Set the power on pin
				res = GPIO.setIOdir(GPIO_POWER_ON, 0, 1)
				if res == 1:
					# wait a moment ...
					MOD.sleep(3)
					ret_value = 0
		if ret_value == 0:
			printI('GPS is ON!')
		else:
			printE('ERROR: unable to power on GPS!!!!!')
		return ret_value
	
	def GPSpowerOff(self):
		"Shutdown the GPS"
		ret_value = -1
		res = -1
		# Set the power on pin
		res = GPIO.setIOdir(GPIO_POWER_ON, 1, 1)
		if res == 1:
			# wait a moment ...
			MOD.sleep(3)
			ret_value = 0
		if ret_value == 0:
			printI('GPS is OFF!')
		else:
			printE('ERROR: unable to shutdown the GPS!!!!!')
		return ret_value
	
	def GPSreset(self):
		"Perform a GPS reset"
		ret_value = -1
		res = -1
		# Set the reset pin to high
		res = GPIO.setIOdir(GPIO_RESET, 1, 1)
		if res == 1:
			# wait a moment ...
			MOD.sleep(3)
			# Set the reset pin to low
			res = GPIO.setIOdir(GPIO_RESET, 0, 1)
			if res == 1:
				# wait a moment ...
				MOD.sleep(3)
				ret_value = 0
		if ret_value == 0:
			printI('GPS reset executed!')
		else:
			printE('ERROR: unable to reset the GPS!!!!!')
		return ret_value
	
	def setPortSpeed(self):
		gps = SER2.set_speed(GPS_PORT_SPEED,'8N1')
	def receiveFromGPS(self):
		"Read raw data from GPS module"
		return SER2.receive(10)

class NMEAsentenceManager:
	"Stores and manages NMEA sentences"
	NMEA_sentences = {'lastRMC':'INVALID!!!'}

	# Instantiate GPS manager
	gps_manager = GPSManager()

	# Instantiate Parser
	msg_parser = NMEAparser()
	
	def __init__(self):
		# Set GPS port speed
		self.gps_manager.setPortSpeed()
		self.gps_manager.GPSpowerOn()
		self.gps_manager.GPSreset()
		pass

	def clearTrail(self):
		"Delete all position from trail buffer"
		printI('clearTrail')
	def getRMCsentence(self):
		"Retrieve a RMC sentence from GPS and insert into trail"
		printI('NMEAsentenceManager.getRMCsentence')
		res = -1
		gpsbuf = ''
		timeout = MOD.secCounter() + 10
		while ((res == -1) and (timeout > MOD.secCounter())):
			gpsbuf = gpsbuf + self.gps_manager.receiveFromGPS()
			res = self.msg_parser.getLastRMCsentence(gpsbuf, self.NMEA_sentences)
			if res == -1:
				printW('...retry...')
				printI(gpsbuf)
		if res == -1:
			printW('Unable to find a complete RMC sentence in xx secs!')
		else:
			printI(self.NMEA_sentences['lastRMC'])
			printI('A complete RMC sentence has been found!')

class SerialPortManager:
	"Read and write from a generic serial port"
	pass

class ModemManager:
	"Send and receive AT commands/responses to/from GSM modem"
	def readRawData(self):
		"Read raw data coming from GSM/GPRS modem"
		printI('Reading data from GSM/GPRS modem :')
		res = ''
		buffer =''
		res = MDM.receive(10)
		timeout = MOD.secCounter() + 5
		while (timeout > MOD.secCounter()):
			buffer = buffer + res
			MOD.sleep(1)
			res = MDM.receive(10)
		if buffer != '':
			printI('\n\r' + buffer)
		return buffer
	
	def readAllSMS(self, waiting_time):
		"Read all SMS received (READ, UNREAD, STORED, etc...)"
		printI('Read all SMS recevied by GSM modem...')
		res = MDM.send('AT+CMGL=ALL\r', 20)
		buffer =''
		res = ''
		error_flag = -1
		timeout = MOD.secCounter() + waiting_time
		# Read ut to timeout elapsed...if there aren't break conditions
		while timeout > MOD.secCounter():
			res = MDM.receive(10)
			MOD.sleep(1)
			buffer = buffer + res
			# Check if the command has been executed correctly
			high_index = buffer.rfind('+CMGL:')
			if(high_index != -1):
				# There is at least one sms to read
				if ((buffer.rfind('OK\r',high_index)) != -1):
					# The command has been executed correctly
					error_flag = 0
					break
			else:
				# There are no sms to read
				high_index = buffer.find('OK\r')
				if(high_index != -1):
					error_flag = 0
					break
		if buffer != '':
			printI('\n\r' + buffer)
		if(error_flag != 0):
			printW('AT+CGMD=ALL not executed correctly!')
		return buffer



	def sendSMS(self, recipient_number, sms_text, waiting_time):
		"Send a SMS"
		printI('Sending SMS with text: '+ sms_text + ' to: ' + recipient_number)
		ret_value = -1
		if(recipient_number[0] == '+'):
			numer_type = '145' # international format
		else:
			numer_type = '129' # national format
			
		res = MDM.send('AT+CMGS='+ recipient_number + ',' + numer_type + '\r', 50)
		if (res == 1):
			# wait for enter text prompt
			prompt_index = -1
			buffer = ''
			timeout = MOD.secCounter() + waiting_time
			while timeout > MOD.secCounter():
				buffer = MDM.receive(10)
				# Check if the prompt has been found
				prompt_index = buffer.find('>')
				if(prompt_index != -1):
					break
				else:
					MOD.sleep(1)
					buffer = ''
			# Enter sms text
			if (prompt_index != -1):
				res = -1
				res = MDM.send(sms_text, 200)
				if (res == 1):
					# Send end character 0x1A
					MOD.sleep(1)
					res = MDM.send('\032', 50)
					if (res == 1):
						# Check for correct execution of the command
						OK_index = -1
						buffer = ''
						timeout = MOD.secCounter() + waiting_time
						while timeout > MOD.secCounter():
							buffer = buffer + MDM.receive(10)
							# Check if the prompt has been found
							OK_index = buffer.find('OK\r')
							if(OK_index != -1):
								ret_value = 0
								printI('SMS has been sent correctly!')
								break
							else:
								MOD.sleep(1)
				else:
					# Try to exit from enter text mode
					MOD.sleep(2)
					res = MDM.send('\032', 50)
		if(ret_value == -1):
			printW('Unable to send SMS!')
		return ret_value
	
	
	def deleteAllSMS(self):
		"Remove all SMS from SIM card"
		printI('Remove all SMS from SIM card')
		res = self.flushATparser(10)
		res = self.sendSimpleATcommand('AT+CMGD=1,4\r', 10, 3)
		return res

	def sendSimpleATcommand(self, command, waiting_time, retry_num):
		"Send a generic AT Command and check for the simple response OK "
		ret_value = -1
		for i in range(1, retry_num):
			printI('Sending the AT Command: ' + command)
			res = ''
			buffer =''
			index = -1
			res = MDM.send(command, 10)
			timeout = MOD.secCounter() + waiting_time
			# Read up to timeout elapsed...if there aren't break conditions
			while timeout > MOD.secCounter():
				res = MDM.receive(10)
				MOD.sleep(1)
				buffer = buffer + res
				# Check if the command has been executed correctly
				index = buffer.find('OK')
				if(index != -1):
					ret_value = 0
					break
			if (ret_value == 0):
				break
			else:
				MOD.sleep(5)
				self.flushATparser(10)
		return ret_value

	def checkRegStatus(self, waiting_time):
		"Verify the status of registration over GSM network"
		printI('Check the GSM registration status...')
		ret_value = -1
		res = ''
		buffer =''
		index = -1
		self.flushATparser(10)
		res = MDM.send('AT+CREG?\r', 10)
		timeout = MOD.secCounter() + waiting_time
		# Read up to timeout elapsed...if there aren't break conditions
		while ((timeout > MOD.secCounter()) and (ret_value == -1)):
			res = MDM.receive(10)
			MOD.sleep(1)
			buffer = buffer + res
			# Check if the command has been executed correctly
			index = buffer.find('+CREG: ')
			if(index != -1):
				# Check if also <mode> and <status> has been read
				if (len(buffer) >= (index + 3)):
					# check the status
					if (((buffer[index + 9]) == '1') or ((buffer[index + 9]) == '5')):
						ret_value = 0
					else:
						# Retry ...
						# Clear variables
						buffer =''
						index = -1
						# Clear AT parser
						self.flushATparser(10)
						# Resend command
						res = MDM.send('AT+CREG?\r', 10)
						printI('Retry to send check registration command')
		if (ret_value == 0):
			printI('Registration OK!')
		else:
			printE('UNABLE TO REGISTER OVER GSM NETWORK!!!')
		return ret_value

	def flushATparser(self, waiting_time):
		"Discard alla data in the AT parser output"
		ret_value = -1
		timeout = MOD.secCounter() + waiting_time
		# Read ut to timeout elapsed...if there aren't break conditions
		while timeout > MOD.secCounter():
			res = MDM.receive(10)
			if res == '':
				ret_value = 0
				break
		return ret_value

	def SMSsettings(self):
		"Set SMS text mode and unsolicited code for incoming sms"
		ret_value = -1
		printI('Set SMS text mode')
		res = self.flushATparser(10)
		if res != -1:
			res = self.sendSimpleATcommand('AT+CMGF=1\r', 10, 3)
			if res != -1:
				printI('Set SMS unsolicited code')
				res = self.flushATparser(10)
				if res != -1:
					res = self.sendSimpleATcommand('AT+CNMI=2,1\r', 10, 3)
					if res != -1:
						main_register.all_values['SMS_settings'] = 'OK'
						ret_value = 0;
		return ret_value

class DataFromModemParser:
	"Retrieve events and command from data coming from GSM/GPRS modem "
	def getFirstValidEvent(self, raw_data):
		"Return the type of the FIRST valid event in the raw data passed as input"
		printI('Parsing data coming from modem from valid event')
		ret_string = -1
		res = raw_data.find('+CMTI:')
		if(res != -1):
			printI('One or more new sms have been received!')
			ret_string = 'NEW_SMS'
		else:
			res = raw_data.find('hello!')
			if(res != -1):
				printI('Alarm detected!')
				ret_string = 'ALARM'
		return ret_string
	
	def getFirstValidSMScommand(self, sms_buffer, dict_of_values):
		"Get the FIRST valid command received via SMS. Command and sender_number will be filled with right values"
		# SMS style : +CMGL: 1,"REC UNREAD","+393348805023","","10/06/14,10:54:01+08"\r\nProva
		printI('Parsing data coming from modem for valid SMS')
		ret_value = -1
		offset_index = 0;
		while (1):
			start_sms_index = sms_buffer.find('+CMGL:', offset_index)  # +
			if start_sms_index != -1:
				cue_index_1 = sms_buffer.find(',',(start_sms_index + 4))  # +CMGL: 1,
				if cue_index_1 != -1:
					cue_index_2 = sms_buffer.find(',',cue_index_1 + 1)  # +CMGL: 1,"REC UNREAD",
					if cue_index_2 != -1:
						apex_index = sms_buffer.find('","',cue_index_2 + 2) # +CMGL: 1,"REC UNREAD","+393348805023"
						if(apex_index != -1):
							dict_of_values['recipient_number'] = sms_buffer[(cue_index_2 + 2):(apex_index)]
							printI('Sender number is '+ dict_of_values['recipient_number'])
							start_text_index = sms_buffer.find('"\r\n', apex_index + 1)
							if(start_text_index != -1):
								end_text_index = sms_buffer.find('\r\n+CMGL:', start_text_index)
								if (end_text_index != -1):
									pass
								else:
									end_text_index = sms_buffer.find('OK', start_text_index)
								if (end_text_index != -1):
									# Search form the valid command
									if((sms_buffer.find('POSITION\r',(start_text_index - 3),(end_text_index + 3))) != -1):
										printI('POSITION command has been found!')
										dict_of_values['command_to_execute'] = 'POSITION'
										ret_value = 0
										break
									elif((sms_buffer.find('NO SLEEP\r',(start_text_index - 3),(end_text_index + 3))) != -1):
										printI('NO SLEEP command has been found!')
										dict_of_values['command_to_execute'] = 'DEL_ALARM'
										ret_value = 0
										break
									elif((sms_buffer.find('SLEEP\r',(start_text_index - 3),(end_text_index + 3))) != -1):
										printI('SLEEP command has been found!')
										dict_of_values['command_to_execute'] = 'SLEEP'
										ret_value = 0
										break
									else:
										printI('Can not find a valid command in string' + sms_buffer[(start_text_index - 3):(end_text_index + 3)] )
										offset_index = end_text_index
								else:
									printW('Can not find a valid end_text_index in string ' + sms_buffer[start_text_index:] )
									break
							else:
								printW('Can not find a valid start_text_index in string ' + sms_buffer[apex_index:] )
								break
						else:
							printW('Can not find a valid apex_index in string ' + sms_buffer[cue_index_2:] )
							break
					else:
						printW('Can not find a valid cue_index_2 in string ' + sms_buffer[cue_index_1:] )
						break
				else:
					printW('Can not find a valid cue_index_1 in string ' + sms_buffer[start_sms_index:] )
					break
			else:
				printI('There are not message to parse in ' + sms_buffer[offset_index:])
				break;
			
		return ret_value


class SMSmanager:
	"Send and receive SMS"
	#smsList = []
	#internal_register = {'recipient_number':'', 'command_to_execute':''}
	
	#Instantiate Modem manager
	modem_manager = ModemManager()

	# Instantiate Parser
	event_parser = DataFromModemParser()
	
	def receiveSMS(self):
		"Check if one or more SMS has been received"
		printI('receiveSMS')
		command = ''
		number = ''
		event_type = ''
		event_buffer = ''
		incoming_sms_buffer = ''
		# Read events from modem
		event_buffer = self.modem_manager.readRawData()
		if event_buffer != '':
			# Parse for a valid event
			event_type = self.event_parser.getFirstValidEvent(event_buffer)
			if event_type == 'NEW_SMS':
				# At least one sms has been received. Read alla sms received.
				incoming_sms_buffer = self.modem_manager.readAllSMS(10)
				if incoming_sms_buffer != '':
					# Parse for the FIRST valid command in incoming sms
					res = self.event_parser.getFirstValidSMScommand(incoming_sms_buffer, main_register.all_values)
					if res != -1:
						# A valid event has been received
						pass
					self.modem_manager.deleteAllSMS()
			elif event_type == 'ALARM':
				# Wake up from alarm detected...reset the alarm!
				main_register.all_values['command_to_execute'] = 'SNOOZE'
			else:
				pass
		else:
			# Check if SMS settings are OK
			if (main_register.all_values['SMS_settings'] == ''):
				res = modem_manager.SMSsettings()
			pass

	def sendSMS(self, recipient_number, sms_text):
		"Send a sms with specific content passed as parameter to the specified number"
		ret_value = -1
		res = modem_manager.flushATparser(5)
		if res != -1:
			res = self.modem_manager.sendSMS(recipient_number, sms_text, 10)
			if res != -1:
				ret_value = 0
		return ret_value


	def clearSMSlist(self):
		"Delete all SMS received from buffer"
		printI('clearSMSlist')
		self.modem_manager.deleteAllSMS()


class AlarmManager:
	"Manage setting and snoozing the alarm and wake up"
	stop_the_alarm = 'AT#WAKE=0\r'
	set_start_time = 'AT+CCLK="04/01/01,10:00:00+01"\r'
	set_wake_up_time = 'AT+CALA="04/01/01,10:03:00+01",0,2,"hello!"\r'
	
	#Instantiate Modem manager
	modem_manager = ModemManager()

	def stopAlarm(self):
		"Stop an Alarm which is ringing and check for network registration status"
		ret_value = -1
		
		# Wake up from shut down!
		
		# Stop the alarm unsolicited code
		res = modem_manager.sendSimpleATcommand(self.stop_the_alarm, 10, 3)
		if res != -1:
			while(1):
				MOD.sleep(3)
				modem_manager.flushATparser(5)
				# Check GSM registration status
				res = modem_manager.checkRegStatus(30)
				if res != -1:
					# Set GSM registration flag in the main register
					main_register.all_values['GSM_reg_status'] = 'OK'
					MOD.sleep(1)
					# Set SMS text mode and unsolicited codes
					res = modem_manager.SMSsettings()
					if res != -1:
						ret_value = 0
						break
				else:
					pass
					# Retry...
		else:
			# IMPLEMENT A GOOD STATEGY!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
			printE('REBOOT?????????????????????????')
			ret_value = -1
			
		return ret_value

	def setAlarm(self):
		"Set the alarm"
		ret_value = -1
		# Reset the time
		res = modem_manager.sendSimpleATcommand(self.set_start_time, 10, 3)
		if res != -1 :
			# Reset the alarm
			res = modem_manager.sendSimpleATcommand(self.set_wake_up_time, 10, 3)
			if res != -1:
				printI('Alarm has been programmed!')
				ret_value = 0
		return ret_value


class CommandManager:
	"Execute commands received by SMS"
	def executeCommand(self, obj_ALARM, obj_NMEA, obj_SMS ):
		"Execute a specific command and send the right message to the recipient, if necessary."
		ret_value = -1
		if(main_register.all_values['command_to_execute'] == 'POSITION'):
			# Send a SMS with the position toward the sender of the request
			printI('Execute command: ' + main_register.all_values['command_to_execute'])
			printI('Use phone number: ' + main_register.all_values['recipient_number'])
			# Check if the recipient_number is valid
			if(main_register.all_values['recipient_number'] != ''):
				# Position required by SMS: send position to SMS sender
				res = -1
				while(res == -1):
					res = obj_SMS.sendSMS(main_register.all_values['recipient_number'], obj_NMEA.NMEA_sentences['lastRMC'])
					if res == -1:
						MOD.sleep(10)
			else:
				# Position NOT required by SMS: it willbe sent toward the default recipient number
				res = -1
				while(res == -1):
					res = obj_SMS.sendSMS(main_register.all_values['def_recipient_num'], obj_NMEA.NMEA_sentences['lastRMC'])
					if res == -1:
						MOD.sleep(10)

		elif(main_register.all_values['command_to_execute'] == 'SLEEP'):
			# Send a SMS with the CONFIRM toward the sender of the request
			printI('Execute command: ' + main_register.all_values['command_to_execute'])
			printI('Use phone number: ' + main_register.all_values['recipient_number'])
			res = -1
			while(res == -1):
				res = obj_SMS.sendSMS(main_register.all_values['recipient_number'], 'SEE YOU LATER...')
				if res == -1:
					MOD.sleep(10)
			
			# Start the count down for the shutdown!!!
			main_register.all_values['go_to_sleep_flag'] = 1
			
			main_register.all_values['shdwn_time'] = MOD.secCounter() + main_register.all_values['wake_up_interval']

		elif(main_register.all_values['command_to_execute'] == 'SNOOZE'):
			# Reset the alarm
			printI('Execute command: ' + main_register.all_values['command_to_execute'])
			printI('Use phone number: ' + main_register.all_values['recipient_number'])
			obj_ALARM.stopAlarm()
			MOD.sleep(3)
			# Position NOT required by SMS: it willbe sent toward the default recipient number
			res = -1
			while(res == -1):
				res = obj_SMS.sendSMS(main_register.all_values['def_recipient_num'], obj_NMEA.NMEA_sentences['lastRMC'])
				if res == -1:
					MOD.sleep(10)
			
			# Start the count down for the shutdown!!!
			main_register.all_values['go_to_sleep_flag'] = 1
			
			main_register.all_values['shdwn_time'] = MOD.secCounter() + main_register.all_values['wake_up_interval']

		elif(main_register.all_values['command_to_execute'] == 'DEL_ALARM'):
			# Send a SMS with the CONFIRM toward the sender of the request
			printI('Execute command: ' + main_register.all_values['command_to_execute'])
			printI('Use phone number: ' + main_register.all_values['recipient_number'])
			res = -1
			while(res == -1):
				if(main_register.all_values['go_to_sleep_flag'] == 1):
					res = obj_SMS.sendSMS(main_register.all_values['recipient_number'], 'OK, the alarm has been deactivated. NO-SLEEPING mode is now active!')
				else:
					res = obj_SMS.sendSMS(main_register.all_values['recipient_number'], 'WARNING, the alarm was not active, stay in NO-SLEEPING mode!')
				if res == -1:
					MOD.sleep(10)
			# Start the count down for the shutdown!!!
			main_register.all_values['go_to_sleep_flag'] = 0
			
			main_register.all_values['shdwn_time'] = 0  

		else:
			pass


		# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
		# COMMON PART
		# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
		
		# Reset the command to execute
		main_register.clear_command()
		main_register.clear_recipient_number()

		time_value = MOD.secCounter()
		# Check if it is time to go to bed...
		if ((main_register.all_values['go_to_sleep_flag'] == 1) and (main_register.all_values['shdwn_time'] <= time_value)):
			# Shutdown the GPS module
			res = -1
			while(res == -1):
				res = obj_NMEA.gps_manager.GPSpowerOff()
				if res == -1:
					MOD.sleep(10)
			# Set the alarm before go to sleep...
			MOD.sleep(3)
			res = -1
			while(res == -1):
				res = obj_ALARM.setAlarm()
				if res == -1:
					MOD.sleep(10)
			MOD.sleep(3)
			
			# Shutdown the module
			
			MDM.send('AT#SHDN\r', 100)
			#main_register.all_values['go_to_sleep_flag'] = 0   # DUMMY!!!!!!!!! JUST FOR DEBUG!!!!

			raise exitOKexception, 'EXIT WITHOUT ERRORS!!!'

			printI('Bye bye ...')
			MOD.sleep(10)
		return ret_value

# #############################

# #############################
# Debug functions
if(1):
	def printI(arg):
		print'I: '+arg
		return 0
	def printW(arg):
		print'W: '+arg
		return 0
	def printE(arg):
		print'E: '+arg
		return 0
else:
	def printI(arg):
		pass
	def printW(arg):
		pass
	def printE(arg):
		pass
# #############################

printI('Hi Gentlemen!')

# #############################
# Initialize of GSM/GPRS module


#SIM status control - to avoid the 'sim busy' error
##print 'SIM Verification Cycle'
##a = MDM.send ('AT+CPBS?\r', 0)
##SIM_status = MDM.receive(10)
##if SIM_status.find("+CPBS")<0:
##	print 'SIM busy! Please wait!\n'
##timeout = MOD.secCounter() + 10
##while ((SIM_status.find("+CPBS:")< 0) and (timeout > MOD.secCounter())) :
##    a = MDM.send ('AT+CPBS?\r', 0)
##    SIM_status = MDM.receive(10)
##    MOD.sleep(2)
##print 'SIM Ready'
# #############################


# #############################
# Initialize data structures
printI('Start initialization procedure')

# Istantiate the main register
main_register = Register()

# Istantiate the alarm manager
alarm_manager = AlarmManager()

# Instantiate the NMEA manager
nmea_manager = NMEAsentenceManager()

# Clear the trail
nmea_manager.clearTrail()

# Instantiate SMS manager
sms_manager = SMSmanager()

# Delete all SMS
#sms_manager.clearSMSlist()

# Instantiate Modem manager
modem_manager = ModemManager()

command_manager = CommandManager()

printI('End of initialization procedure')
# ############################

# ############################
try:
	try:
		while(1):
			nmea_manager.getRMCsentence()
			MOD.sleep(10);
	##			command_manager.executeCommand(alarm_manager, nmea_manager, sms_manager)
	##			MOD.sleep(10);
			sms_manager.receiveSMS()
			MOD.sleep(10);
			command_manager.executeCommand(alarm_manager, nmea_manager, sms_manager)
			printI('End of cycle')
	except IOError:
		printE('I/O ERROR!')
finally:
	printE('GENERIC ERROR!')

printI('Bye bye!')

